-----------------------------------------------------------------------------------------------
-- Client Lua Script for ArmorSwap
-- (c) Sjentre (Orias)
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Unit"
require "Item"
require "GameLib"
require "Apollo"
require "math"
require "string"

-----------------------------------------------------------------------------------------------
-- ArmorSwap Module Definition
-----------------------------------------------------------------------------------------------
local ArmorSwap = {} 

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local tEquippedNames = 
{
	"Augment",
	"Chest",
	"Feet",
	"Gadget",
	"Legs",
	"Shields",
	"Shoulder",
	"Hands",
	"Head",
	"Implant",
	"WeaponPrimary",
	"Primary Weapon",
	"Weapon Attachment",
	"Support System"
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ArmorSwap:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.tArmorSet1 = {}
	self.tArmorSet2 = {}
	self.tArmorSet3 = {}
	self.tArmorSet4 = {}
	
    return o
end

function ArmorSwap:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "ArmorSwap"
	local tDependencies = {}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- ArmorSwap Overrides
-----------------------------------------------------------------------------------------------

function ArmorSwap:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("ArmorSwap.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
end

function ArmorSwap:OnDocumentReady()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ArmorSwapForm", nil, self)

		self.wndBag = self.wndMain:FindChild("BagWindow")
		
		self.iTop = self.iTop or 10
		self.iLeft = self.iLeft or 197
		self.wndMain:FindChild("EditBox1"):SetText(self.sButton1)
		self.wndMain:FindChild("EditBox2"):SetText(self.sButton2)
		self.wndMain:FindChild("EditBox3"):SetText(self.sButton3)
		self.wndMain:FindChild("EditBox4"):SetText(self.sButton4)
		self.wndMain:FindChild("btnArmor1"):SetText(self.sButton1)
		self.wndMain:FindChild("btnArmor2"):SetText(self.sButton2)
		self.wndMain:FindChild("btnArmor3"):SetText(self.sButton3)
		self.wndMain:FindChild("btnArmor4"):SetText(self.sButton4)
		self.wndMain:SetAnchorOffsets(self.iLeft, self.iTop, self.iLeft + 222, self.iTop + 194)
		
		self.wndMain:Show(true)
		self.wndBag:Show(false)
		self.xmlDoc = nil
				
		Apollo.RegisterEventHandler("PlayerEquippedItemChanged", "OnEquipped", self)
	end
end

function ArmorSwap:OnConfigure()
	self.wndMain:Show(true)
	self:ShowConfigure()
end

function ArmorSwap:OnEquipped()
	if self.IsShowing then
		self:LoadArmorSet(self.ArmorSet)
	end
end

function ArmorSwap:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	local iLeft, iTop = self.wndMain:GetAnchorOffsets()

	return {
		tArmorSet1 = self.tArmorSet1,
		tArmorSet2 = self.tArmorSet2,
		tArmorSet3 = self.tArmorSet3,
		tArmorSet4 = self.tArmorSet4,
		sButton1Name = self.wndMain:FindChild("EditBox1"):GetText(),
		sButton2Name = self.wndMain:FindChild("EditBox2"):GetText(),
		sButton3Name = self.wndMain:FindChild("EditBox3"):GetText(),
		sButton4Name = self.wndMain:FindChild("EditBox4"):GetText(),
		iTop = iTop,
		iLeft = iLeft
		}
end

function ArmorSwap:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	self.tArmorSet1 = tData.tArmorSet1 or {}
	self.tArmorSet2 = tData.tArmorSet2 or {}
	self.tArmorSet3 = tData.tArmorSet3 or {}
	self.tArmorSet4 = tData.tArmorSet4 or {}
	self.sButton1 = tData.sButton1Name or "Armor 1"
	self.sButton2 = tData.sButton2Name or "Armor 2"
	self.sButton3 = tData.sButton3Name or "Armor 3"
	self.sButton4 = tData.sButton4Name or "Armor 4"
	self.iTop = tData.iTop
	self.iLeft = tData.iLeft	
end

-----------------------------------------------------------------------------------------------
-- ArmorSwap Functions
-----------------------------------------------------------------------------------------------

function ArmorSwap:ShowConfigure()
	self.wndBag:Show(false)
	self:ToggleConfigure(true)	
end

function ArmorSwap:ToggleConfigure(bShow)
	if bShow == nil then
		bShow = not self.wndMain:FindChild("btnSave1"):IsVisible()
	end

	self.wndMain:FindChild("btnSave1"):Show(bShow)
	self.wndMain:FindChild("btnSave2"):Show(bShow)
	self.wndMain:FindChild("btnSave3"):Show(bShow)
	self.wndMain:FindChild("btnSave4"):Show(bShow)
	self.wndMain:FindChild("EditBox1"):Show(bShow)
	self.wndMain:FindChild("EditBox2"):Show(bShow)
	self.wndMain:FindChild("EditBox3"):Show(bShow)
	self.wndMain:FindChild("EditBox4"):Show(bShow)

	if bShow then
		local t 
		
		for i = 2, 5, 1 do
			t = self.wndMain:GetPixieInfo(i)
			t.cr = {1,1,1,1}
			self.wndMain:UpdatePixie(i, t)
		end

	else
		local t 
		
		for i = 2, 5, 1 do
			t = self.wndMain:GetPixieInfo(i)
			t.cr = {1,1,1,0}
			self.wndMain:UpdatePixie(i, t)
		end
	end

end

function ArmorSwap:IsIn(a, b)
	if b ~= nil then
		for index, value in ipairs(b) do
			if value == a then
				return true
			end
		end
	end
	
	return false 
end

function ArmorSwap:SaveArmorSet(iSetNumber)
	local unitPlayer = GameLib.GetPlayerUnit()
	local tSet = {}
	
	if unitPlayer ~= nil then
		local tItems = unitPlayer:GetEquippedItems()

		for idx, itemCurr in ipairs(tItems) do
			if self:IsIn(itemCurr:GetSlotName(), tEquippedNames) then
				tSet[itemCurr:GetSlotName()] = itemCurr:GetName()
			elseif self:IsIn(itemCurr:GetItemTypeName(), tEquippedNames) then
				tSet[itemCurr:GetItemTypeName()] = itemCurr:GetName()
			end
		end
	end

	if iSetNumber == 1 then
		self.tArmorSet1 = tSet
	elseif iSetNumber == 2 then
		self.tArmorSet2 = tSet
	elseif iSetNumber == 3 then
		self.tArmorSet3 = tSet
	elseif iSetNumber == 4 then
		self.tArmorSet4 = tSet
	end

	unitPlayer = nil
end

function ArmorSwap:LoadArmorSet()
	local tLoading
	local tCurrent = {}
	local tSwap = {}
	local unitPlayer = GameLib.GetPlayerUnit()

	if unitPlayer ~= nil then
		local tItems = unitPlayer:GetEquippedItems()

		for idx, itemCurr in ipairs(tItems) do
			if self:IsIn(itemCurr:GetSlotName(), tEquippedNames) then
				tCurrent[itemCurr:GetSlotName()] = itemCurr:GetName()
			elseif self:IsIn(itemCurr:GetItemTypeName(), tEquippedNames) then
				tCurrent[itemCurr:GetItemTypeName()] = itemCurr:GetName()
			end
		end
	end
	
	if self.ArmorSet == 1 then
		tLoading = self.tArmorSet1
	elseif self.ArmorSet == 2 then
		tLoading = self.tArmorSet2
	elseif self.ArmorSet == 3 then
		tLoading = self.tArmorSet3
	elseif self.ArmorSet == 4 then
		tLoading = self.tArmorSet4
	end

	if tLoading ~= nil then
		for i, v in ipairs(tEquippedNames) do
			if tLoading[v] ~= tCurrent[v] and tLoading[v] then
				table.insert(tSwap, tLoading[v])
			end
		end
	end

	if #tSwap > 0 then
		self.IsShowing = true
		self:ShowBag(tSwap)
	else
		self.IsShowing = false
		self.wndBag:Show(false)
	end
	
	self:ToggleConfigure(false)
end

function ArmorSwap:ShowBag(tSwap)
	local nLeft, nTop = self.wndMain:GetAnchorOffsets()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	-- -----------------------------------
	--TODO: Once API works change this...
	self.HasAddedPixies = false
	self.TotalBagItems = #unitPlayer:GetInventoryItems()
	self.ItemsFound = {}
	
	self.wndBag:FindChild("btnHide1"):Show(false)
	self.wndBag:FindChild("btnHide2"):Show(false)
	self.wndBag:FindChild("btnHide3"):Show(false)
	self.wndBag:FindChild("btnHide4"):Show(false)
	self.wndBag:FindChild("btnHide5"):Show(false)
	self.wndBag:FindChild("btnHide6"):Show(false)
	-- -----------------------------------
	
	self.wndBag:SetSort(true)
	self.wndBag:SetItemSortComparer(function(...) return self:ItemSort(tSwap, ...) end)
	self.wndBag:Show(true)

end

function ArmorSwap:ItemSort (itemNames, itemLeft, itemRight)
	local strLeftName 
	local strRightName

	if itemLeft then 
		strLeftName = itemLeft:GetName()
		
		if not self:IsIn(strLeftName, self.ItemsFound) then
			table.insert(self.ItemsFound, strLeftName)
		end
	end

	if itemRight then
		strRightName = itemRight:GetName()

		if not self:IsIn(strLeftName, self.ItemsFound) then
			table.insert(self.ItemsFound, strRightName)
		end
	end
	
	if true then -- not self.HasAddedPixies and #self.ItemsFound == self.TotalBagItems then
		local itemsToShow = 0
		self.HasAddedPixies = true
		
		if #self.ItemsFound > 0 then
			for i = 1, #self.ItemsFound do
				if self:IsIn(self.ItemsFound[i], itemNames) then
					itemsToShow = itemsToShow + 1
				end
			end
		end
		
		self.wndBag:FindChild("btnHide1"):Show(itemsToShow <= 0)
		self.wndBag:FindChild("btnHide2"):Show(itemsToShow <= 1)
		self.wndBag:FindChild("btnHide3"):Show(itemsToShow <= 2)
		self.wndBag:FindChild("btnHide4"):Show(itemsToShow <= 3)
		self.wndBag:FindChild("btnHide5"):Show(itemsToShow <= 4)
		self.wndBag:FindChild("btnHide6"):Show(itemsToShow <= 5)
	end
	
	if itemLeft == itemRight then
		return 0
	end

	if itemLeft and itemRight == nil then
		if self:IsIn(strLeftName, itemNames) then
			return -1
		else
			return 1
		end
	end
	
	if itemLeft == nil and itemRight then
		if self:IsIn(strRightName, itemNames) then
			return 1
		else
			return -1
		end
	end

	if self:IsIn(strLeftName, itemNames) and ArmorSwap:IsIn(strRightName, itemNames) then
		return 0
	elseif self:IsIn(strLeftName, itemNames) then
		return -1
	elseif self:IsIn(strRightName, itemNames) then
		return 1
	else
		return 0
	end
end

-----------------------------------------------------------------------------------------------
-- ArmorSwapForm Events
-----------------------------------------------------------------------------------------------

function ArmorSwap:OnOK()
	self.wndMain:Close()
end

function ArmorSwap:OnCancel()
	self.wndMain:Close()
end

function ArmorSwap:btnLoad_Click( wndHandler, wndControl, eMouseButton )
	local name = wndControl:GetName()
	
	if name == "btnArmor1" then
		self.ArmorSet = 1
	elseif name == "btnArmor2" then
		self.ArmorSet = 2
	elseif name == "btnArmor3" then
		self.ArmorSet = 3
	elseif name == "btnArmor4" then
		self.ArmorSet = 4
	end
	
	self:LoadArmorSet()
end

function ArmorSwap:btnSave_Click( wndHandler, wndControl, eMouseButton )
	local name = wndControl:GetName()
	
	if name == "btnSave1" then
		self:SaveArmorSet(1)
	elseif name == "btnSave2" then
		self:SaveArmorSet(2)
	elseif name == "btnSave3" then
		self:SaveArmorSet(3)
	elseif name == "btnSave4" then
		self:SaveArmorSet(4)
	end
end

function ArmorSwap:btnConfigure_Click( wndHandler, wndControl, eMouseButton)
	self.wndBag:Show(false)
	self:ToggleConfigure()
end

function ArmorSwap:btnClose_Click( wndHandler, wndControl, eMouseButton )
	self.wndMain:Show(false)
	self.wndBag:Show(false)
end

function ArmorSwap:TextBox_Changed( wndHandler, wndControl, strText)
	local name = wndControl:GetName()
	
	if name == "EditBox1" then
		self.wndMain:FindChild("btnArmor1"):SetText(strText)
	elseif name == "EditBox2" then		
		self.wndMain:FindChild("btnArmor2"):SetText(strText)
	elseif name == "EditBox3" then		
		self.wndMain:FindChild("btnArmor3"):SetText(strText)
	elseif name == "EditBox4" then		
		self.wndMain:FindChild("btnArmor4"):SetText(strText)
	end
end

function ArmorSwap:TextBox_EnterPressed( wndHandler, wndControl, strText )
	wndControl:ClearFocus()
end

-----------------------------------------------------------------------------------------------
-- ArmorSwap Instance
-----------------------------------------------------------------------------------------------

local ArmorSwapInst = ArmorSwap:new()
ArmorSwapInst:Init()
